# Proteinortho

 Proteinortho is a tool to detect orthologous genes within different species. For doing so, it compares similarities of given gene sequences and clusters them to find significant groups. The algorithm was designed to handle large-scale data and can be applied to hundreds of species at one. Details can be found in <a href="https://bmcbioinformatics.biomedcentral.com/articles/10.1186/1471-2105-12-124">Lechner et al., BMC Bioinformatics. 2011 Apr 28;12:124.</a>
To enhance the prediction accuracy, the relative order of genes (synteny) can be used as additional feature for the discrimination of orthologs. The corresponding extension, namely PoFF (manuscript in preparation), is already build in Proteinortho. 

# New Features of Proteinortho Version 6!

  - Implementation of various Blast alternatives for step (-step=2): Diamond, Last, Topaz, Rapsearch, Blat, Ublast and Usearch
  - Multithreading support for the clustering step (-step=3)
  - Integration of the LAPACK Fortran Library for a faster clustering step (-step=3)
  - Integration of the bitscore weights in the connectivity calculation for more data dependant splits (-step=3)
<details>
  <summary>Minor new features: (Click to expand)</summary>

  - OrthoXML support
  - Test routines
  - Heuristics for connectivity calculation (-step=3)
</details>

# Continuous Integration

The badge 
[![pipeline status](https://gitlab.com/paulklemm_PHD/proteinortho/badges/master/pipeline.svg)](https://gitlab.com/paulklemm_PHD/proteinortho/commits/master) indicates the current status of the continuous integration (CI) among various platforms (ubuntu, centos, debian, fedora) and GNU c++ versions (5, 6, latest)
The whole git repository gets deployed on a clean docker imager (gcc:latest,gcc:5,ubuntu:latest,debian:latest,centos:latest) and compiled (make all) and tested (make test). The badge is green only if all test are passed.

# Table of Contents
1. [Installation](#installation)
2. [Synopsis and Description](#synopsis)
3. [Options/Parameters](#options)
4. [PoFF synteny extension](#poff)
5. [Output description](#output)
6. [Examples](#examples)

# Installation
#### 1. Prerequisites

Proteinortho uses standard software which is often installed already or is part of then package repositories and can thus easily be installed.
<details>
  <summary>To run Proteinortho, you need: (Click to expand)</summary>
  
   - NCBI BLAST+ or NCBI BLAST legacy (to test this, type tblastn (BLAST+) or blastall (BLAST+) in the command line)
   - (optional) the following programs
   
     - Diamond (to test this, type 'diamond' in the command line)
     - Last (to test this, type 'lastal' in the command line)
     - Rapsearch (to test this, type 'rapsearch' in the command line)
     - Topaz (to test this, type 'topaz' in the command line)
     - usearch (to test this, type 'usearch' in the command line)
     - ublast (is part of usearch)
     - blat (to test this, type 'blat' in the command line)
   - Perl v5.08 or higher (to test this, type perl -v in the command line)
   - Python v2.6.0 or higher to include synteny analysis (to test this, type 'python -V' in the command line) 
   - 
</details>

<br>

The sources come with a precompiled version of Proteinortho for 64bit Linux. If you want/need to recompile and install it, you will also need
<details>
  <summary>To compile Proteinortho, you need: (Click to expand)</summary>

   - GNU make (to test this, type 'make' in the command line)
   - GNU g++ v4.1 or higher (to test this, type 'g++ --version' in the command line) 
   - openmp (to test this, type 'g++ -fopenmp' in the command line) 
   - gfortran (to test this, type 'whereis gfortran' in the command line)
   - CMake for compiling LAPACK (to test this, type 'cmake' in the command line) 
   
</details>

<br>

#### 2. Building and installing from source

  Fetch the latest source code archive downloaded from <a href="https://gitlab.com/paulklemm_PHD/proteinortho/-/archive/master/proteinortho-master.zip">here</a> 
<details>
  <summary>or from here (Click to expand)</summary>

  > <code>git clone https://gitlab.com/paulklemm_PHD/proteinortho</code>
  >
  > <code>wget https://gitlab.com/paulklemm_PHD/proteinortho/-/archive/master/proteinortho-master.zip</code>
</details>
<br>
  
  - Extract the files e.g. via `tar -xzvf proteinortho*.tar.gz</code> or <code>unzip proteinortho*.zip`
  - Change directory into the extracted folder e.g. via <code>cd proteinortho*</code>
  - You can now run <code>./proteinortho6.pl</code> directly.
  - If you want to recompile and install Proteinortho, type make followed <code>make install</code> or <code>make install INSTALLDIR=/home/paul/bin</code> if you dont have root privileges.
  - For a version without lapack: <code>make USELAPACK=FALSE</code>.
  - For a variation that uses an allready installed version of lapack: <code>make USEPRECOMPILEDLAPACK=TRUE</code>.
  - In any case, run <code>make test</code> to make sure Proteinortho works as expected. The output should look like this:
 

<details>
  <summary>'make' successful output (Click to expand)</summary>
<pre>
[  0%] Prepare proteinortho_clustering ...
[  5%] Extracting the LAPACK library (if not src/lapack is not present)
[ 10%] Compiling the LAPACK library (if not src/lapack/build/lib/liblapack.a is not present)
-- The Fortran compiler identification is GNU 4.8.4
-- The C compiler identification is GNU 4.8.4
-- Check for working Fortran compiler: /usr/bin/f95
-- Detecting Fortran compiler ABI info
-- Detecting Fortran compiler ABI info - done
-- Checking whether /usr/bin/f95 supports Fortran 90
-- Checking whether /usr/bin/f95 supports Fortran 90 -- yes
-- Check for working C compiler: /usr/bin/cc
-- Check for working C compiler: /usr/bin/cc -- works
-- Detecting C compiler ABI info
-- Detecting C compiler ABI info - done
-- Detecting C compile features
-- Detecting C compile features - done
-- Setting build type to 'Release' as none was specified.
-- Checking if build type is 'Coverage'
-- Checking if build type is 'Coverage': 0
-- Build tests: OFF
-- Reducing RELEASE optimization level to O2
-- Looking for Fortran NONE - found
-- Looking for Fortran INT_CPU_TIME - found
-- Looking for Fortran EXT_ETIME - not found
-- Looking for Fortran EXT_ETIME_ - not found
-- Looking for Fortran INT_ETIME - found
-- --> Will use second_INT_ETIME.f and dsecnd_INT_ETIME.f as timing function.
-- Build deprecated routines: OFF
-- Build single precision real: ON
-- Build double precision real: ON
-- Build single precision complex: ON
-- Build double precision complex: ON
-- Using supplied NETLIB BLAS implementation
-- Using supplied NETLIB LAPACK implementation
-- Configuring done
-- Generating done
-- Build files have been written to: /home/paul/Documents/proteinortho/src/lapack/build
[ 20%] Building **proteinortho_clustering** with LAPACK
g++ -std=c++11 -Wall -O3 -fopenmp -flto  -o src/proteinortho_clustering src/proteinortho_clustering.cpp -Isrc/lapack/build/include/ -Lsrc/lapack/build/lib/ -llapack -lblas -lgfortran
[ 25%] Building **graphMinusRemovegraph**
g++ -Wall -O3 -std=gnu++11 -Wno-unused-result  -o src/graphMinusRemovegraph src/graphMinusRemovegraph.cpp
[ 50%] Building **cleanupblastgraph**
g++ -std=c++11 -Wall -O3 -std=gnu++11 -Wno-unused-result  -o src/cleanupblastgraph src/cleanupblastgraph.cpp
[ 75%] Building **po_tree**
gcc  -o src/po_tree src/po_tree.c
Everything is compiled with no errors.
</pre>
</details>

<details>
  <summary>'make test' successful output (Click to expand)</summary>
<pre>
Everything is compiled with no errors.
[TEST] 1. basic proteinortho6.pl -step=2 tests
 [1/11] -p=blastp+ test: passed
 [2/11] -p=blastp+ synteny (PoFF) test: passed
 [3/11] -p=diamond test: passed
 [4/11] -p=diamond (--moresensitive) test (subpara): passed
 [5/11] -p=lastp (lastal) test: passed
 [6/11] -p=topaz test: passed
 [7/11] -p=usearch test: passed
 [8/11] -p=ublast test: passed
 [9/11] -p=rapsearch test: passed
 [10/11] -p=blatp (blat) test: passed
 [11/11] -p=mmseqsp (mmseqs) test: passed
[TEST] 2. -step=3 tests (src/proteinortho_clustering)
 [1/2] various test functions of src/proteinortho_clustering (-test): passed
 [2/2] Compare results of 'with lapack' and 'without lapack': passed
[TEST] Clean up all test files...
[TEST] All tests passed
</pre>
</details>

<br>

# SYNOPSIS
  proteinortho6.pl [options] \<fasta file\>
  OR
  proteinortho [options] \<fasta file\>

# DESCRIPTION
  **proteinortho** is a tool to detect orthologous genes within different
  species. For doing so, it compares similarities of given gene sequences
  and clusters them to find significant groups. The algorithm was designed
  to handle large-scale data and can be applied to hundreds of species at
  one. Details can be found in Lechner et al., BMC Bioinformatics. 2011 Apr
  28;12:124. To enhance the prediction accuracy, the relative order of genes
  (synteny) can be used as additional feature for the discrimination of
  orthologs. The corresponding extension, namely PoFF (manuscript in
  preparation), is already build in Proteinortho.

  Proteinortho assumes, that you have all your gene sequences in FASTA
  format either represented as amino acids or as nucleotides. The source
  code archive contains some examples, namely C.faa, E.faa, L.faa, M.faa
  located in the test/ directory. **default Proteinortho assumes amino**
  **acids and thus uses diamond** (-p=diamond) to compare sequences. If you have
  nucleotide sequences, you need to change this by adding the parameter
  -p=blastn+ (or some other algorithm). (In case you have only have NCBI
  BLAST legacy installed, you need to tell this too - either by adding
  -p=blastp or -p=blastn respectively.) The full command for the example
  files would thus be proteinortho6.pl -project=test test/C.faa test/E.faa
  test/L.faa test/M.faa. Instead of naming the FASTA files one by one, you
  could also supply test/*.faa as argument. Please note that the parameter
  -project=test is optional. With this, you can set the prefix of the output
  files generated by Proteinortho. If you skip the project parameter, the
  default project name will be myproject.

# OPTIONS
 **Main parameters**
 
   - **--project**=name (default: myproject)
    prefix for all resulting file names

   - **--cpus**=number (default: all available)
    the number of processors to use (multicore/processor support)

  - **--ram**=number (default: 90% of free memory)
    maximal used ram threshold for LAPACK and the input graph in MB

  - **--verbose**={0,1,2} (default: 0)
    verbose level. 1:keeps you informed about the progress

  - **--temp**=directory(.)
    path to the temporary files

  - **--force**
    forces the recalculation of the blast results in any case

  - **--clean**
    removes all unnecessary files afterwards

  - **--step**={0,1,2,3} (default: 0)
    0 -> all. 1 -> prepare blast (build db). 2 -> run all-versus-all
    blast. 3 -> run the clustering.

 **Search options (step 1-2)**
  (output: <myproject>.blast-graph)

<details>
  <summary>(Click to expand)</summary>
  
  - **--p**=algorithm (default: diamond)

    <details>
      <summary>show all algorithms (Click to expand)</summary>
      
        - **blastn**,**blastp**,**tblastx**,**blastn+**,**blastp+**,**tblastx+** : standard blast
        family. The suffix 'n' or 'p' indicates nucleotide or protein version
        (of the input files).
    
        - **diamond** : Only for protein files! standard diamond procedure and for
        genes/proteins of length >40 with the additional --sensitive flag
    
        - **lastn**,**lastp** : lastal. -n : dna files, -p protein files (BLOSUM62
        scoring matrix)!
    
        - **rapsearch** : Only for protein files!
    
        - **mmseqsp**,**mmseqsn** : mmseqs2.
    
        - **topaz** : Only for protein files!
    
        - **usearch** : usearch_local procedure with -id 0 (minimum identity
        percentage).
    
        - **ublast** : usearch_ublast procedure.
    
        - **blatp**,**blatn** : blat. -n : dna files, -p protein files
    </details>
    <br>

  - **--e**=evalue (default: 1e-05)
    apply selfblast, detects paralogs without orthologs

  - **--selfblast**
    E-value for blast

  - **--sim**=float (default: 0.95)
    min. similarity for additional hits

  - **--identity**=number (default: 25)
    min. percent identity of best blast hits

  - **--cov**=number (default: 50)
    min. coverage of best blast alignments in %

  - **--subpara**='options'
    additional parameters for the search tool example -subpara='-seg no'
    or -subpara='--more-sensitive' for diamond
</details>
<br>

 **Synteny options (optional, step 2)**
  (output: <myproject>.ffadj-graph, <myproject>.poff-graph)

<details>
  <summary>(Click to expand)</summary>
  
  - **--synteny**
    activate PoFF extension to separate similar by contextual adjacencies
    (requires .gff for each .fasta)

  - **--dups**=number (default: 0)
    PoFF: number of reiterations for adjacencies heuristic, to determine
    duplicated regions

  - **--cs**=number (default: 3)
    PoFF: Size of a maximum common substring (MCS) for adjacency matches

  - **--alpha**=number (default: .5)
    PoFF: weight of adjacencies vs. sequence similarity
</details>
<br>

 **Clustering options (step 3)**
  (output: <myproject>.proteinortho, <myproject>.proteinortho-graph)

<details>
  <summary>(Click to expand)</summary>
      
  - **--mcl**
    enables the mcl algorithm for clustering instead of power iteration or
    lapacks routine. (needs mcl to be installed, call 'mcl' to test if you
    installed mcl correctly)

  - **--singles**
    report singleton genes without any hit

  - **--purity**=float (default: 1e-7)
    avoid spurious graph assignments

  - **--conn**=float (default: 0.1)
    min. algebraic connectivity

  - **--nograph**
    do not generate .graph file (pairwise orthology relations)

  - **--xml**
    do generate an orthologyXML file (.xml)

  - **--exactstep3**
    perform step 3 without the k-mere heuristic (much slower for huge
    datasets but more precise)
</details>
<br>

 **Misc options**
 
<details>
  <summary>(Click to expand)</summary>
      
  - **--desc**
    write description files (for NCBI FASTA input only)

  - **--binpath**=directory (default: PATH)
    path to your local executables (blast, diamond, mcl, ...)

  - **--debug**
    gives detailed information for bug tracking

 **Large compute jobs**
  - **---jobs**=M/N
    If you want to involve multiple machines or separate a Proteinortho
    run into smaller chunks, use the -jobs=**M**/**N** option. First, run
    'proteinortho6.pl -steps=1 ...' to generate the indices. Then you can
    run 'proteinortho6.pl -steps=2 -jobs=**M**/**N** ...' to run small chunks
    separately. Instead of **M** and **N** numbers must be set representing the
    number of jobs you want to divide the run into (**M**) and the job
    division to be performed by the process. E.g. to divide a Proteinortho
    run into 4 jobs to run on several machines, use
</details>
<br>

# PoFF

  The PoFF extension allows you to use the relative order of genes (synteny)
  as an additional criterion to disentangle complex co-orthology relations.
  To do so, add the parameter -synteny. You can use it to either come closer
  to one-to-one orthology relations by preferring synthetically conserved
  copies in the presence of two very similar paralogs (default), or just to
  reduce noise in the predictions by detecting multiple copies of genomic
  areas (add the parameter -dups=3). Please note that you need additional
  data to include synteny, namely the gene positions in GFF3 format. 
  AsProteinortho is primarily made for proteins, it will only accept GFF
  entries of type CDS (column #3 in the GFF-file). The attributes column
  (#9) must contain Name=GENE IDENTIFIER where GENE IDENTIFIER corresponds
  to the respective identifier in the FASTA format. It may not contain a
  semicolon (;)! Alternatively, you can also set ID=GENE IDENTIFIER. Example
  files are provided in the source code archive. Hence, we can run
  proteinortho6.pl -project=test -synteny test/A1.faa test/B1.faa test/E1.faa
  test/F1.faa to add synteny information to the calculations. Of course,
  this only makes sense if species are sufficiently similar. You won't gain
  much when comparing e.g. bacteria with fungi. When the analysis is done
  you will find an additional file in your current working directory, namely
  test.poff. This file is equivalent to the .proteinortho file (above) but
  can be considered more accurate as synteny was involved for its
  construction.
  
# Output
 **BLAST Search (step 1-2)**
 
<details>
  <summary>myproject.blast-graph (Click to expand)</summary>
  
    filtered raw blast data based on adaptive reciprocal best blast
    matches (= reciprocal best match plus all reciprocal matches within a
    range of 95% by default) The first two rows are just comments
    explaining the meaning of each row. Whenever a comment line (starting
    with #) follows, it indicates results comparing the two species is
    about to follow. E.g. #M.faa L.faa tells that the next lines represent
    results for species M and L. All matches are reciprocal matches. If
    e.g. a match for M_15 L_15 is shown, L_15 M_15 exists implicitly.
    E-Values and bit scores for both directions are given behind each
    match.

      # file_a    file_b 
      # a   b     evalue_ab     bitscore_ab   evalue_ba     bitscore_ba 
      # E.faa     C.faa
      E_10  C_10  3.8e-124    434.9   2.8e-126    442.2
      E_11  C_11  5.9e-51 190.7   5.6e-50 187.61
 </details>
 <br>
  
 **Clustering (step 3)**

<details>
  <summary>myproject.proteinortho-graph (Click to expand)</summary>
    clustered myproject.blast-graph. Its connected components are
    represented in myproject.proteinortho. The format is the same as the
    blast-graph (see above).

      # file_a    file_b
      # a   b     evalue_ab     bitscore_ab   evalue_ba     bitscore_ba
      # E.faa     C.faa
      E_10  C_10  3.8e-124    434.9   2.8e-126    442.2
      E_11  C_11  5.9e-51 190.7   5.6e-50 187.6
 </details>
 <br>
  
 <details>
  <summary> myproject.proteinortho (Click to expand)</summary>
    The connected components. The first line starting with #is a comment
    line indicating the meaning of each column for each of the following
    lines which represent an orthologous group each. The very first column
    indicates the number of species covered by this group. The second
    column indicates the number of genes included in the group. Often,
    this number will equal the number of species, meaning that there is a
    single ortholog in each species. If the number of genes is bigger than
    the number of species, there are co-orthologs present. The third
    column gives rise to the algebraic connectivity of the respective
    group. Basically, this indicates how densely the genes are connected
    in the orthology graph that was used for clustering. A connectivity of
    1 indicates a perfect dense cluster with each gene similar to each
    other gene. By default, Proteinortho splits each group into two more
    dense subgroups when the connectivity is below 0.1

      # Species   Genes   Alg.-Conn.    C.faa   C2.faa  E.faa   L.faa   M.faa
      2   5     0.16  *     *     *     L_643,L_641   M_649,M_640,M_642
      3   6     0.138   C_164,C_166,C_167,C_2   *     *     L_2   M_2
      2   4     0.489   *     *     *     L_645,L_647   M_644,M_646
 </details>
 <br>
  
 **POFF (-synteny)**
 
  The synteny based graph files (myproject.ffadj-graph and
  myproject.poff-graph) have two additional columns: same_strand and
  simscore. The first one indicates if two genes from a match are located at
  the same strands (1) or not (-1). The second one is an internal score
  which can be interpreted as a normalized weight ranging from 0 to 1 based
  on the respective e-values. Moreover, a second comment line is followed
  after the species lines, e.g.

    # M.faa L.faa
    # Scores: 4   39    34.000000     39.000000

  <details>
  <summary>myproject.ffadj-graph (Click to expand)</summary>
    filtered blast data based on adaptive reciprocal best blast matches
    and synteny (only if -synteny is set)
 </details>
 <br>
  
  <details>
  <summary>myproject.poff-graph (Click to expand)</summary>
    clustered ffadj graph. Its connected components are represented in
    myproject.poff (only if -synteny is set)
 </details>
 <br>
  

# EXAMPLES
 **Calling proteinortho**
  Sequences are typically given in plain fasta format like the files in
  test/

  test/C.faa:
  
    >C_10
    VVLCRYEIGGLAQVLDTQFDMYTNCHKMCSADSQVTYKEAANLTARVTTDRQKEPLTGGY
    HGAKLGFLGCSLLRSRDYGYPEQNFHAKTDLFALPMGDHYCGDEGSGNAYLCDFDNQYGR
    ...

   test/E.faa:
   
    >E_10
    CVLDNYQIALLRNVLPKLFMTKNFIEGMCGGGGEENYKAMTRATAKSTTDNQNAPLSGGF
    NDGKMGTGCLPSAAKNYKYPENAVSGASNLYALIVGESYCGDENDDKAYLCDVNQYAPNV
    ...

  To run proteinortho for these sequences, simply call

    proteinortho6.pl test/C.faa test/E.faa test/L.faa test/M.faa

  To give the outputs the name 'test', call

    proteinortho6.pl -project=test test/*faa

  To use blast instead of the default diamond, call

    proteinortho6.pl -project=test -p=blastp+ test/*faa


# Hints
  Using .faa to indicate that your file contains amino acids and .fna to
  show it contains nucleotides makes life much easier.

  Sequence IDs must be unique within a single FASTA file. Consider renaming
  otherwise. Note: Till version 5.15 sequences IDs had to be unique among
  the whole dataset. Proteinortho now keeps track of name and species to
  avoid the necessissity of renaming.

  You need write permissions in the directory of your FASTA files as
  Proteinortho will create blast databases. If this is not the case,
  consider using symbolic links to the FASTA files.

  The directory src contains useful tools, e.g. grab_proteins.pl which
  fetches protein sequences of orthologous groups from Proteinortho output
  table


# Kmere Heuristic


## Example 1

In the following example a huge blast graph is used for step 3 (clustering). 
The first connected component contains 7410694 nodes, hence the kmere heuristic is activated.
Since the fiedler vector would result in a good split, the kmere heuristic is then deactivated immediatly.

<details>
  <summary>as fallback (Click to expand)</summary>
    ...
    [CRITICAL WARNING]   Failed to partition subgraph with 6929 nodes into (6929,0,0) sized groups, now using kmere heuristic as fall-back.
    ...
</details>

<details>
<summary>working example for large graphs (Click to expand)</summary>
    ...
    17:32:15 [DEBUG] (kmere-heuristic) The current connected component is so large that the k-mere heuristic can be used. First: Testing if a normal split would result in a good partition (|.|>20%) of the CC.
     [WARNING] (kmere-heuristic) A normal split would NOT result in a good partition (|.|>20%) of the CC, therefore  the k-mere heuristic is now used. The current connected component will be split in 3.85373 (= number of proteins <6929> / ( n
    odes per species <1> * number of species <1798>)) groups greedily accordingly to the fiedler vector.
    ...
</details>
    
<details>
<summary>example for large graphs, where kmere is tested but not needed (Click to expand)</summary>
    ...
    20:27:07 [DEBUG] (kmere-heuristic) The current connected component is so large that the k-mere heuristic can be used. First: Testing if a normal split would result in a good partition (|.|>20%) of the CC.
    20:27:09 [DEBUG] (kmere-heuristic) A normal split would result in a good partition (|.|>20%) of the CC, therefore returning now to the normal algorithm (no k-mere heuristic).
    ...
</details>

# Critical-Edge Heuristic

The critical edge heuristic is another fall-back algorithm for the clustering algortihm.
An edge of a connected compent is called critical if upon the removal, the connectivity is lost.
Given the edge (x,y) of the connected component C: 
The BFS algorithm can determine (in linear time) if x and y are connected in C\{(x,y)}.

The following example shows how a connected component of 66933 nodes (with 32000 maxnode threshold) is decomposed in a 39105 node connected component by removing 210 critcal edges (in 7 seconds).
<details>
  <summary>(Click to expand)</summary>
  
    09:21:16 [DEBUG]   Debug level 1
    09:21:16 [DEBUG]   Parsing file blast-graph
    09:21:34 [DEBUG]   I know 78 species with 578202 proteins and 5378032 edges in sum
    09:21:34 [DEBUG]   Maximumum number of nodes for connectivity calculations is 32000
    09:21:34 [DEBUG]   Sorting known species
    09:21:35 [DEBUG]   Clustering
    09:21:35 [DEBUG] Found connected component: 73 proteins (ID: 0)
    09:21:35 [DEBUG] Found connected component: 4195 proteins (ID: 2)
    09:21:35 [DEBUG] Found connected component: 117 proteins (ID: 4)
    09:21:35 [DEBUG] Found connected component: 89 proteins (ID: 6)
    09:21:35 [DEBUG] Found connected component: 66933 proteins (ID: 8)
    [WARNING]  Found a very large connected component that contains 66933>32000 (maxnodes) elements. This behavior can be adjusted using -maxnodes, still you better consider a higher e-value or coverage thresholds. Now using the Critical-Heuristic: try to identify and remove critical edges.
    [WARNING] (critical edge heuristic) Found and removed 210 critical edge(s).
    [WARNING]  Now restarting the partition algorithm.
    09:21:43 [DEBUG] Found connected component: 73 proteins (ID: 0)
    09:21:43 [DEBUG] Found connected component: 4195 proteins (ID: 2)
    09:21:43 [DEBUG] Found connected component: 117 proteins (ID: 4)
    09:21:43 [DEBUG] Found connected component: 89 proteins (ID: 6)
    09:21:43 [DEBUG] Found connected component: 39105 proteins (ID: 8)
    (...)
</details>

<details>
  <summary>(Click to expand)</summary>
  
    15:30:35 [DEBUG]   Debug level 1
    15:30:35 [DEBUG]   Parsing file Gn100_d0.089.sav.sif
    15:30:35 [DEBUG]   I know 1 species with 99 proteins and 269 edges in sum
    15:30:35 [DEBUG]   Maximumum number of nodes for connectivity calculations is 51
    15:30:35 [DEBUG]   Sorting known species
    15:30:35 [DEBUG] Found connected component: 99 proteins (ID: 0)
    [WARNING]  Found a very large connected component that contains 99>51 (maxnodes) elements. This behavior can be adjusted using -maxnodes, still you better consider a higher e-value or coverage thresholds. Now using the Critical-Heuristic: try to identify and remove critical edges.
    [WARNING] (critical edge heuristic)  Found and removed 1 critical edge(s).
    [WARNING]  Now restarting the partition algorithm.
    15:30:35 [DEBUG] Found connected component: 50 proteins (ID: 0)
    15:30:35 [DEBUG] Found connected component: 49 proteins (ID: 6)
    15:30:35 [DEBUG] Found: 2 connected components in total, continue now with the calculation of the algebraic connectivities.
    15:30:35 [DEBUG]   Performing now the power iteration of all CC with more than 8192 nodes.
    15:30:35 [DEBUG]   Remaining 2 connected components are prepared for algebraic connectivity calculation using LAPACK.
    15:30:35 [DEBUG]   The algebraic connectivity calculation are now done, continue now with the evaluation of the scores.
    15:30:35 [DEBUG]     Connectivity was 0.0207996
    15:30:35 [DEBUG]     Connectivity was 0.0110796
    (...)
</details>


# ONLINE INFORMATION
  For download and online information, see
  <https://www.bioinf.uni-leipzig.de/Software/proteinortho/>
  or
  <https://gitlab.com/paulklemm_PHD/proteinortho>

# REFERENCES
  Lechner, M., Findeisz, S., Steiner, L., Marz, M., Stadler, P. F., &
  Prohaska, S. J. (2011). Proteinortho: detection of (co-) orthologs in
  large-scale analysis. BMC bioinformatics, 12(1), 124.


